<?php

namespace IQBall\App\Controller;

use IQBall\App\Session\MutableSessionHandle;
use IQBall\App\Session\SessionHandle;
use IQBall\App\ViewHttpResponse;
use IQBall\Core\Http\HttpResponse;
use IQBall\Core\Model\TacticModel;
use IQBall\Core\Model\TeamModel;

class UserController {
    private TacticModel $tactics;
    private ?TeamModel $teams;


    /**
     * @param TacticModel $tactics
     * @param TeamModel|null $teams
     */
    public function __construct(TacticModel $tactics, ?TeamModel $teams = null) {
        $this->tactics = $tactics;
        $this->teams = $teams;
    }

    /**
     * @param SessionHandle $session
     * @return ViewHttpResponse the home page view
     */
    public function home(SessionHandle $session): ViewHttpResponse {
        $limitNbTactics = 5;

        $user = $session->getAccount()->getUser();

        $lastTactics = $this->tactics->getLast($limitNbTactics, $user->getId());
        $allTactics = $this->tactics->getAll($user->getId());

        $allTacticsShared = $this->tactics->getAllTacticShared($user->getId());
        if(isset($allTacticsShared)) {
            foreach ($allTacticsShared as $tactic) {
                if(!in_array($tactic, $allTactics)) {
                    array_push($allTactics, $tactic);
                }
            }
        }

        if ($this->teams != null) {
            $teams = $this->teams->getAll($user->getId());
        } else {
            $teams = [];
        }

        return ViewHttpResponse::react("views/Home.tsx", [
            "lastTactics" => $lastTactics,
            "allTactics" => $allTactics,
            "teams" => $teams,
            "user" => $user,
        ]);
    }

    /**
     * @return ViewHttpResponse account settings page
     */
    public function settings(SessionHandle $session): ViewHttpResponse {
        return ViewHttpResponse::react("views/Settings.tsx", []);
    }

    public function disconnect(MutableSessionHandle $session): HttpResponse {
        $session->destroy();
        return HttpResponse::redirect("/");
    }
}
